package ai.pai.lensman.session;

import android.os.AsyncTask;

import com.android.common.executors.ThreadExecutor;
import com.android.common.utils.LogHelper;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import ai.pai.lensman.bean.PhotoBean;
import ai.pai.lensman.box.BoxUrlContainer;
import ai.pai.lensman.utils.HttpPostTask;

/**
 * Created by chengzhenyu on 2016/7/7.
 */
public class SessionInteractor {

    private String sessionId;
    private String lensmanId;
    private Timer timer;
    private SessionListener listener;
    private HttpPostTask sessionStartTask;
    private HttpPostTask sessionEndTask;
    private HttpPostTask fetchThumbnailTask;
    private HttpPostTask fetchOriginTask;

    private static final String TAG = "SessionInteractor";

        public SessionInteractor(String lensmanId,String sessionId, SessionListener listener) {
        this.lensmanId = lensmanId;
        this.listener = listener;
        this.sessionId = sessionId;

    }

    public interface SessionListener {

        void onSessionStartSuccess(String session);
        void onSessionStartError(String session);
        void onSessionPhotoCaptured(ArrayList<PhotoBean> bean);
        void onSessionEnd(String session);
    }

    public void startSession(){
        cancelTask(sessionStartTask);
        HashMap<String,String> params = new HashMap<>();
        params.put("lensman",lensmanId);
        params.put("session",sessionId);
        sessionStartTask = new HttpPostTask(params){
            @Override
            protected boolean parseResponse(String response) {
                LogHelper.d(TAG,"startSession get response string = "+response);
                try{
                    JSONObject json = new JSONObject(response);
                    int status = json.getInt("status");
                    if(status == 200){
                        return true;
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
                return false;
            }

            @Override
            protected void onPostFail() {
                super.onPostFail();
                listener.onSessionStartError(sessionId);
            }

            @Override
            protected void onPostSuccess() {
                super.onPostSuccess();
                listener.onSessionStartSuccess(sessionId);
                startCapture();
            }
        };
        sessionStartTask.executeOnExecutor(ThreadExecutor.getInstance().getExecutor(), BoxUrlContainer.SESSION_START_URL);
    }

    public void startCapture() {
        if(timer!=null){
            timer.cancel();
            timer = null;
        }
        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                fetchThumbnailTask();
            }
        },1000,5000);
    }

    private void fetchThumbnailTask(){
        cancelTask(fetchThumbnailTask);
        HashMap<String,String> params = new HashMap<>();
        params.put("lensman",lensmanId);
        params.put("session",sessionId);
        fetchThumbnailTask = new HttpPostTask(params){
            ArrayList<PhotoBean> photoList = new ArrayList<>();
            @Override
            protected boolean parseResponse(String response) {
                LogHelper.d(TAG,"fetchThumbnailTask get response string = "+response);
                try{
                    JSONObject json = new JSONObject(response);
                    int status = json.getInt("status");
                    if(status == 200){
                        JSONObject data = json.getJSONObject("data");
                        JSONArray files = data.getJSONArray("files");
                        if(files!=null && files.length()>0){
                            for(int k = 0 ; k< files.length();k++){
                                JSONObject file = files.getJSONObject(k);
                                PhotoBean bean = new PhotoBean();
                                bean.photoName = file.getString("name");
                                bean.photoId = file.getLong("id");
                                bean.photoPath = file.getString("path");
                                bean.captureTime = bean.photoId;
                                bean.isRawPhoto = false;
                                bean.isUploaded = false;
                                bean.sessionId = sessionId;
                                photoList.add(bean);
                            }
                        }
                        return true;
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
                return false;
            }

            @Override
            protected void onPostFail() {
                super.onPostFail();
            }

            @Override
            protected void onPostSuccess() {
                super.onPostSuccess();
                listener.onSessionPhotoCaptured(photoList);
            }
        };
        fetchThumbnailTask.executeOnExecutor(ThreadExecutor.getInstance().getExecutor(), BoxUrlContainer.FETCH_THUMBNAIL_URL);
    }

    public void endSession(){
        cancelTask(sessionEndTask);
        HashMap<String,String> params = new HashMap<>();
        params.put("lensman",lensmanId);
        params.put("session",sessionId);
        sessionEndTask = new HttpPostTask(params);
        sessionEndTask.executeOnExecutor(ThreadExecutor.getInstance().getExecutor(), BoxUrlContainer.SESSION_END_URL);
        if(timer!=null){
            timer.cancel();
            timer = null;
        }
        listener.onSessionEnd(sessionId);
    }

    private void cancelTask(HttpPostTask task){
        if(task==null){
            return;
        }
        if(task.getStatus()== AsyncTask.Status.RUNNING){
            task.cancel(true);
        }
    }

}
